@@ -1,8 +1,9 @@
 import httpx
 import ujson
 import asyncio
-from typing import Dict, List, Optional, Tuple, Union
+from typing import Any, Dict, List, Optional, Tuple, Union
 from functools import wraps
+from idac_sdk.models.vpn_config import VPNConfig, VPNType
 
 from idac_sdk.types import IDACRequestStatus, IDACRequestType
 from idac_sdk.log import logger
@@ -15,6 +16,7 @@ from idac_sdk.errors import (
     IncorrectWantedStateError,
     NoControllerError,
     NoIdError,
+    UnknownVPNType,
 )
 from idac_sdk.models.request_state import RequestState
 from idac_sdk.session_data import SessionData
@@ -47,6 +49,7 @@ class IDACRequest:
     controller: IDACController
     uuid: Optional[str]
     user_agent: str
+    vpn: VPNConfig
 
     def __init__(
         self,
@@ -54,6 +57,7 @@ class IDACRequest:
         session_data: Optional[SessionData] = None,
         controller: Optional[IDACController] = None,
         user_agent: Optional[str] = None,
+        vpn: Optional[VPNConfig] = None,
     ) -> None:
         """
         IDACRequest Object
@@ -69,6 +73,7 @@ class IDACRequest:
             controller (Optional[IDACController], optional): IDACCOntroller object.
                 Defaults to None.
             user_agent (Optional[str], optional): User-Agent string. Defaults to None.
+            vpn (Optional[VPNConfig], optional): VPN options
         """
         if not uuid:
             # no UUID -> new request, need to set session_data and controller
@@ -92,15 +97,65 @@ class IDACRequest:
         else:
             self.controller = controller
 
+        self.vpn = vpn or self.controller.vpn
+
     async def check_controller_auth(self) -> None:
         """Tells controller to request auth token if auth type is DCLOUD_SESSION"""
-        logger.debug(f"Checking auth. Type is {self.controller.auth_type}")
-        if self.controller.auth_type == IDACAuthType.DCLOUD_SESSION:
+        logger.debug(f"Checking auth. Type is {self.controller.auth.type}")
+        if self.controller.auth.type == IDACAuthType.DCLOUD_SESSION.name:
             assert self.session_data, "Session data not set"
             token = await self.controller.get_auth_token(
                 self.session_data.get("creds"), self.session_data.get("datacenter")
             )
-            logger.debug(f"Token is: {token}")
+            logger.debug(f"dCloud token is: {token}")
+        elif self.controller.auth.type == IDACAuthType.WORKER.name:
+            token = await self.controller.get_auth_token()
+            logger.debug(f"Worker token is: {token}")
+
+    def add_vpn(self, where: dict[str, Any]) -> dict[str, Any]:
+        """Adds VPN parameters to request
+
+        Args:
+            where (dict[str, Any]): Where VPN params should be added
+
+        Raises:
+            UnknownVPNType: Raised if unknown VPN type configured
+
+        Returns:
+            dict[str, Any]: updated dictionary with VPN parameters
+        """
+        logger.debug("Adding VPN parameters: %s", self.vpn)
+        if ("idacVpn" in where) or (self.vpn.type == VPNType.none):
+            return where
+
+        if self.vpn.type == VPNType.vpod:
+            # adding params for vPod VPN. `datacenter`, `vpod` and `anycpwd` should be loaded from session.xml
+            where.update({"idacVpn": "dcloud-vpod"})
+        elif self.vpn.type == VPNType.request or self.vpn.type == VPNType.explicit:
+            # adding params for `request` - explicit VPN settings
+            where.update(
+                {
+                    "idacVpn": "request",
+                    "idacVpnHost": self.vpn.params.host,
+                    "idacVpnUsername": self.vpn.params.username,
+                    "idacVpnPassword": self.vpn.params.password,
+                }
+            )
+        elif self.vpn.type == VPNType.secure_repo:
+            # adding params for `secure_repo` - VPN settings will be loaded from secure repo
+            where.update(
+                {
+                    "idacVpn": "secureRepo",
+                    "idacVpnSecureRepoSpace": self.vpn.params.secure_repo_space,
+                    "idacVpnSecureRepoFile": self.vpn.params.secure_repo_file,
+                    "idacVpnSecureRepoBlock": self.vpn.params.secure_repo_block,
+                    "idacVpnSecureRepoKey": self.vpn.params.secure_repo_key,
+                }
+            )
+        else:
+            raise UnknownVPNType("Unknown VPN type provided")
+
+        return where
 
     @check_controller
     async def create(
@@ -130,13 +185,16 @@ class IDACRequest:
         logger.debug(f"API: {method} to {api}")
         logger.debug(f"Headers are: {headers}")
 
+        # convert SessionData to dict and populate VPN parameters
+        data = self.add_vpn(self.session_data.dict(exclude_none=True))
+
         if method == "POST":
             # put data in body if POST
-            kwargs = {"json": self.session_data.dict(exclude_none=True)}
+            kwargs = {"json": data}
             headers.update({"Content-Type": "application/json"})
         else:
             # put data in query string if GET
-            kwargs = {"params": self.session_data.dict(exclude_none=True)}
+            kwargs = {"params": data}
 
         redirect = None
         st = None
