@@ -48,3 +48,15 @@ class IncorrectMinutesValue(Exception):
 
 class NoIDACConfig(Warning):
     ...
+
+
+class OldConfigFoundException(Exception):
+    ...
+
+
+class OldConfigFoundWarning(Warning):
+    ...
+
+
+class UnknownVPNType(Exception):
+    ...
